/*
 * Copyright (C) 2010 Xisco Faulí Tarazona
 *
 * This file is part of Domestica.
 * 
 * Domestica is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * Domestica is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domestica; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

package beans;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.ArrayList;
import java.util.List;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.URL;

import objects.Device;
import objects.StateImage;
import objects.commandButton;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import org.primefaces.event.SlideEndEvent;

public class DevicesPage {

	private Device currentDevice;
	private StateImage currentImage;
	private List<Entry<String, String>> properties;
	private List<commandButton> buttons;
	private Boolean pageRendered;
	private Boolean propertiesRendered;
	private Boolean sliderRendered;
	private Boolean inputTextRendered;
	private int inputTextValue;

	private int sliderNumber;
	private Map<String, StateImage> currentStateMap;

	private Collection deviceCol;

	public void getInformation(String currentItem) {
		// Set current device and load all it data
		Map session = FacesContext.getCurrentInstance().getExternalContext()
				.getSessionMap();
		if (deviceCol == null) {
			deviceCol = (Collection) session.get("devices");
		}

		pageRendered = true;

		Iterator iDevice = deviceCol.iterator();

		while (iDevice.hasNext()) {
			Device d = (Device) iDevice.next();
			if (currentItem.equals(d.getId())) {
				currentDevice = d;
				if (!d.getProperties().isEmpty()) {
					properties = new ArrayList<Entry<String, String>>(d
							.getProperties().entrySet());
					propertiesRendered = true;
				} else {
					propertiesRendered = false;
				}
				if (!d.getStateImageList().isEmpty()) {
					currentStateMap = (Map<String, StateImage>) session
							.get("currentStateMap");
					currentImage = currentStateMap.get(currentDevice.getId());
				} else {
					currentImage = new StateImage("Unknown",
							"no_available.jpg", "no_available.jpg");
				}
			}
		}
		_createButtons();
	}

	public void _createButtons() {
		// Method called for create / update the action buttons
		if (buttons == null) {
			buttons = new ArrayList<commandButton>();
		} else {
			buttons.clear();
		}
		sliderRendered = false;
		inputTextRendered = false;

		if (currentDevice.getType().equals("DEVICE_TYPE_SWITCH")) {
			boolean disabled;
			if (currentImage.getStateValue().equals("off")) {
				disabled = true;
			} else {
				disabled = false;
			}
			buttons.add(new commandButton("On", !disabled,
					"Click to turn this device on"));
			buttons.add(new commandButton("Off", disabled,
					"Click to turn this device off"));
			return;
		} else if (currentDevice.getType().equals(
				"DEVICE_TYPE_PERCENTUAL_DIMMER")) {
			sliderRendered = true;
			int min = Integer
					.parseInt(currentImage.getStateValue().split("-")[0]);
			int max = Integer
					.parseInt(currentImage.getStateValue().split("-")[1]);
			sliderNumber = (int) (min + max) / 2;
			boolean disabled;
			if (min == 0) {
				disabled = true;
			} else {
				disabled = false;
			}
			buttons.add(new commandButton("On", !disabled,
					"Click to turn this device on"));
			buttons.add(new commandButton("Off", disabled,
					"Click to turn this device off"));
			return;
		} else if (currentDevice.getType().equals("DEVICE_TYPE_SENSOR")) {
			buttons.add(new commandButton("Refresh", false,
					"Click here to refresh the device"));
		} else if (currentDevice.getType().equals("DEVICE_TYPE_PULSE")) {
			buttons.add(new commandButton("Pulse", false,
					"Click here to pulse the device"));
		} else if (currentDevice.getType().equals(
				"DEVICE_TYPE_VERTICAL_MOVEMENT")) {
			buttons.add(new commandButton("Up", false,
					"Click here to move the device up"));
			buttons.add(new commandButton("Down", false,
					"Click here to move the device down"));
			buttons.add(new commandButton("Stop", true,
					"Click here to stop the current action"));
		} else if (currentDevice.getType().equals("DEVICE_TYPE_ANGULAR_DIMMER")) {
			inputTextRendered = true;
			buttons.add(new commandButton("Set Value", false,
					"Click here to set the Angular Value"));
		} else if (currentDevice.getType().equals("DEVICE_TYPE_HEX_DIMMER")) {
			inputTextRendered = true;
			buttons.add(new commandButton("Summit", false,
					"Click here to set the Hex Value"));
		}

	}

	public void onSlideEnd(SlideEndEvent event) {
		// Method called when the slider is moved
		Map session = FacesContext.getCurrentInstance().getExternalContext()
				.getSessionMap();
		sliderNumber = event.getValue();

		currentImage = _getCurrentImage(sliderNumber);
		_createButtons();
		_updateStateMap(session);

		String DeviceType = "es.upv.pros.tunnel.interfaces.IActuatorPercentualDimmer";
		String Action = "setPercentualValue&Value="
				+ String.valueOf(sliderNumber);
		// _requestHTTP(DeviceType, Action);
	}

	public void onButtonClicked(ActionEvent event) {
		// Method called when an action button is clicked
		Map session = FacesContext.getCurrentInstance().getExternalContext()
				.getSessionMap();

		String selectedButton = (String) event.getComponent().getAttributes()
				.get("selectedButton");

		String DeviceType = "";
		String Action = "";
		if (currentDevice.getType().equals("DEVICE_TYPE_SWITCH")) {
			Iterator i = currentDevice.getStateImageList().iterator();
			while (i.hasNext()) {
				StateImage s = (StateImage) i.next();
				if (!s.getStateValue().equals(
						currentStateMap.get(currentDevice.getId())
								.getStateValue())) {
					currentImage = s;
				}
			}
			_createButtons();

			_updateStateMap(session);

			DeviceType = "es.upv.pros.tunnel.interfaces.IActuatorSwitch";
			Action = "switch" + selectedButton;
		} else if (currentDevice.getType().equals(
				"DEVICE_TYPE_PERCENTUAL_DIMMER")) {
			Object[] array = (Object[]) currentDevice.getStateImageList()
					.toArray();
			if (selectedButton.equals("On")) {
				int size = array.length;
				currentImage = (StateImage) array[size - 1];
			} else {
				currentImage = (StateImage) array[0];
			}
			_createButtons();

			_updateStateMap(session);

			DeviceType = "es.upv.pros.tunnel.interfaces.IActuatorPercentualDimmer";
			Action = selectedButton.toLowerCase();

		} else if (currentDevice.getType().equals("DEVICE_TYPE_SENSOR")) {
			DeviceType = "es.upv.pros.tunnel.interfaces.ISensor";
			Action = "getState";

		} else if (currentDevice.getType().equals("DEVICE_TYPE_PULSE")) {
			DeviceType = "es.upv.pros.tunnel.interfaces.IActuatorPulse";
			Action = "pulse";

		} else if (currentDevice.getType().equals(
				"DEVICE_TYPE_VERTICAL_MOVEMENT")) {
			DeviceType = "es.upv.pros.tunnel.interfaces.IActuatorVerticalMovement";
			if (selectedButton.equals("Stop")) {
				Action = selectedButton.toLowerCase();
			} else {
				Action = "move" + selectedButton;
			}
		} else if (currentDevice.getType().equals("DEVICE_TYPE_ANGULAR_DIMMER")) {
			if (inputTextValue >= 0 && inputTextValue <= 360) {
				currentImage = _getCurrentImage(inputTextValue);
				DeviceType = "es.upv.pros.tunnel.interfaces.IActuatorAngularDimmer";
				Action = "setAngularValue&Value="
						+ String.valueOf(inputTextValue);
			} else {
				return;
			}
		} else if (currentDevice.getType().equals("DEVICE_TYPE_HEX_DIMMER")) {
			if (inputTextValue >= 0 && inputTextValue <= 255) {
				currentImage = _getCurrentImage(inputTextValue);
				DeviceType = "es.upv.pros.tunnel.interfaces.IActuatorHexDimmer";
				Action = "setHexValue&Value=" + String.valueOf(inputTextValue);
			} else {
				return;
			}
		}

		// _requestHTTP(DeviceType, Action);
	}

	public StateImage _getCurrentImage(int value) {
		StateImage aux = new StateImage();
		Iterator iter = currentDevice.getStateImageList().iterator();
		while (iter.hasNext()) {
			StateImage s = (StateImage) iter.next();
			int min = Integer.parseInt(s.getStateValue().split("-")[0]);
			int max = Integer.parseInt(s.getStateValue().split("-")[1]);
			if (min <= value && max >= value) {
				aux = s;
			}
		}
		return aux;
	}

	public void onImageClicked(ActionEvent event) {
		// method called when a device image is clicked from the map of the
		// house
		String selectedDevice = (String) event.getComponent().getAttributes()
				.get("selectedDevice");
		getInformation(selectedDevice);
	}

	public void _updateStateMap(Map session) {
		// Method that update the session attributes with the device state
		currentStateMap.put(currentDevice.getId(), currentImage);

		try {
			session.put("currentStateMap", currentStateMap);
		} catch (Exception E) {
		}
	}

	public void _requestHTTP(String DeviceType, String Action) {
		// Method that connect with the real devices

		Map session = FacesContext.getCurrentInstance().getExternalContext()
				.getSessionMap();

		String server = (String) session.get("server");
		String requestUrl = "http://" + server + "/muteServlet/" + "?DeviceID="
				+ currentDevice.getId() + "&DeviceType=" + DeviceType
				+ "&Action=" + Action;
		System.out.println(requestUrl);

		try {
			URL url = new URL(requestUrl.toString());
			BufferedReader in = new BufferedReader(new InputStreamReader(url
					.openStream()));
			String state = in.readLine();
			in.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public Boolean getInputTextRendered() {
		return inputTextRendered;
	}

	public void setInputTextRendered(Boolean inputTextRendered) {
		this.inputTextRendered = inputTextRendered;
	}

	public int getInputTextValue() {
		return inputTextValue;
	}

	public void setInputTextValue(int inputTextValue) {
		this.inputTextValue = inputTextValue;
	}

	public List<commandButton> getButtons() {
		return buttons;
	}

	public void setButtons(List<commandButton> buttons) {
		this.buttons = buttons;
	}

	public Boolean getSliderRendered() {
		return sliderRendered;
	}

	public void setSliderRendered(Boolean sliderRendered) {
		this.sliderRendered = sliderRendered;
	}

	public StateImage getCurrentImage() {
		return currentImage;
	}

	public void setCurrentImage(StateImage currentImage) {
		this.currentImage = currentImage;
	}

	public Boolean getPropertiesRendered() {
		return propertiesRendered;
	}

	public void setPropertiesRendered(Boolean propertiesRendered) {
		this.propertiesRendered = propertiesRendered;
	}

	public int getSliderNumber() {
		return sliderNumber;
	}

	public void setSliderNumber(int sliderNumber) {
		this.sliderNumber = sliderNumber;
	}

	public Boolean getPageRendered() {
		return pageRendered;
	}

	public void setPageRendered(Boolean pageRendered) {
		this.pageRendered = pageRendered;
	}

	public List<Entry<String, String>> getProperties() {
		return properties;
	}

	public void setProperties(List<Entry<String, String>> properties) {
		this.properties = properties;
	}

	public Device getCurrentDevice() {
		return currentDevice;
	}

	public void setCurrentDevice(Device currentDevice) {
		this.currentDevice = currentDevice;
	}
}
