/*
 * Copyright (C) 2010 Xisco Faulí Tarazona
 *
 * This file is part of Domestica.
 * 
 * Domestica is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * Domestica is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domestica; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
*/

package beans;

import java.util.*;

import objects.Coordenates;
import objects.Device;
import objects.Image;
import objects.Location;
import objects.StateImage;
import objects.SubLocation;
import objects.LocationDevice;

import org.primefaces.model.DefaultTreeNode;
import org.primefaces.model.TreeNode;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;

//XML Libraries
import org.jdom.*;
import org.jdom.input.*;


public class LoadData {
	
	
	private Collection DeviceCollection;
	private Collection LocationCollection;
	private Map<String,TreeNode> LocationTreeMap;
	private Map<String,TreeNode> DevicesTreeMap;
	private Map<String,StateImage> currentStateMap;
	private boolean read = false;

	public void init(){
		
		Map session = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();

		if(read == false){
		try{
	    	    SAXBuilder builder=new SAXBuilder(false);
				
				ServletContext context = (ServletContext) FacesContext.getCurrentInstance().getExternalContext().getContext();
				
				String filepath = context.getRealPath("xml/ScenarioSpecificationV0.3.xml");
				
				//build up a memory tree
				Document doc=builder.build(filepath);
				
				//Initialize variables
				LocationCollection = new ArrayList();
				DeviceCollection = new ArrayList();
				LocationTreeMap = new HashMap<String,TreeNode>();
				DevicesTreeMap = new HashMap<String,TreeNode>();

				
				currentStateMap = new HashMap<String,StateImage>();

				Element devicesSpecification = doc.getRootElement().getChild("InstallationDevices");

				//All the children whose have Device as a name
				List devices = devicesSpecification.getChildren("Device");
				
				Iterator iDevices = devices.iterator();
				while (iDevices.hasNext()){
					Element device = (Element)iDevices.next();
					
					//Children of Device
					String id = device.getChild("ID").getAttributeValue("ID");
					String tunnel = device.getChild("Tunnel").getAttributeValue("tunelName");
					String type = device.getChild("Type").getAttributeValue("deviceType");
					String profile = device.getChild("Profile").getAttributeValue("profile");
					String address = device.getChild("Address").getAttributeValue("address");
					String dataType = device.getChild("DataType").getAttributeValue("dataType");
					
					Collection StateImageList = new ArrayList();
					String platformName = "";
					
					if(!device.getChild("Platform").getChildren().isEmpty()){

						platformName = device.getChild("Platform").getChild("PlatformName").getAttributeValue("name");
						
						List StateList = device.getChild("Platform").getChildren("StateImage");
						Iterator iListA = StateList.iterator();
				
						while(iListA.hasNext()){
							Element elementState=(Element)iListA.next();
							String StateValue = elementState.getChild("StateValue").getAttributeValue("state");
							String ImageFile = elementState.getChild("ImageFile").getAttributeValue("URL");
							String ThumbnailFile = elementState.getChild("ThumbnailFile").getAttributeValue("URL");
							StateImage stateimage = new StateImage(StateValue,ImageFile,ThumbnailFile);
							StateImageList.add(stateimage);
						}
						//PROVISIONAL
						if(!StateImageList.isEmpty()){
							currentStateMap.put(id, getStateValue(StateImageList));
						}
					}
					
					Map<String,String> propertiesMap = new HashMap<String,String>();
					
					//Device's Properties
					if(!device.getChild("Properties").getChildren().isEmpty()){
						List proList = device.getChild("Properties").getChildren("Property");
						
						Iterator iListB = proList.iterator();
						while(iListB.hasNext()){
							Element elementProperty=(Element)iListB.next();
							propertiesMap.put(elementProperty.getAttributeValue("literal"), elementProperty.getAttributeValue("value"));
						
						} 
					} 
					Device dev = new Device(id,tunnel,type,profile,address,dataType,platformName, StateImageList,propertiesMap);
				    DeviceCollection.add(dev);
				    
					DevicesTreeMap.put(id, new DefaultTreeNode("device",id,null));
				}
				
				Element positionSpecification = doc.getRootElement().getChild("Locations");
				//All the children which have Location as a name
				List locations = positionSpecification.getChildren("Location");
				
				Iterator iLocations = locations.iterator();
				while(iLocations.hasNext()){
					Element location = (Element)iLocations.next();
					String id = location.getChild("ID").getAttributeValue("ID");
					String name = location.getChild("Name").getAttributeValue("name");
					String superLocation = location.getChild("SuperLocationIDRef").getAttributeValue("superLocationID");
					Element e = location.getChild("Image");
					Image image = new Image(e.getAttributeValue("URL"),e.getAttributeValue("PosX"),e.getAttributeValue("PosY"),e.getAttributeValue("Width"),e.getAttributeValue("Height"));
					
					
					Collection  subLocationList = new ArrayList();
					
					if(!location.getChildren("Sublocations").isEmpty()){
						List subLocations = location.getChild("Sublocations").getChildren("Sublocation");
						Iterator isubLocation = subLocations.iterator();
						while (isubLocation.hasNext()){
							Element subLocation = (Element)isubLocation.next();
							String LocationIDRef = subLocation.getChild("LocationIDRef").getAttributeValue("locationID");
							Element c = subLocation.getChild("ClickableArea");
							Coordenates ClickableArea = new Coordenates(c.getAttributeValue("PosX"),c.getAttributeValue("PosY"),c.getAttributeValue("Width"),c.getAttributeValue("Height"));
							Element t = subLocation.getChild("Thumbnail");
							Image Thumbnail = new Image(t.getAttributeValue("URL"),t.getAttributeValue("PosX"),t.getAttributeValue("PosY"),t.getAttributeValue("Width"),t.getAttributeValue("Height"));
							
							SubLocation subLoc = new SubLocation(LocationIDRef,ClickableArea,Thumbnail);
							subLocationList.add(subLoc);
						}
						
						
					}
					
					Collection<LocationDevice>  LocationDeviceList = new ArrayList<LocationDevice>();
					
					if(!location.getChildren("LocationDevices").isEmpty()){
						List locationDevices = location.getChild("LocationDevices").getChildren("LocationDevice");
						Iterator ilocationDevice = locationDevices.iterator();
						while (ilocationDevice.hasNext()){
							Element locationDevice = (Element)ilocationDevice.next();
							String DeviceIDRef = locationDevice.getChild("DeviceIDRef").getAttributeValue("deviceID");
							Element c = locationDevice.getChild("ImagePosition");
							Coordenates ImagePosition = new Coordenates(c.getAttributeValue("PosX"),c.getAttributeValue("PosY"),c.getAttributeValue("Width"),c.getAttributeValue("Height"));
							Element t = locationDevice.getChild("ThumbnailPosition");
							Coordenates ThumbnailPosition = new Coordenates(t.getAttributeValue("PosX"),t.getAttributeValue("PosY"),t.getAttributeValue("Width"),t.getAttributeValue("Height"));
							
							LocationDevice subDev = new LocationDevice(DeviceIDRef,ImagePosition,ThumbnailPosition);
							LocationDeviceList.add(subDev);
						}
						
					}
					LocationCollection.add(new Location(id,name,superLocation,image,subLocationList,LocationDeviceList));
					
					//Create a Map of TreeNodes in order to creates the nodes of the tree of the webpage
					LocationTreeMap.put(id, new DefaultTreeNode("location",id,null));  
				}

				session.put("locations",LocationCollection );
				session.put("locationNodes", LocationTreeMap);
				session.put("devices", DeviceCollection);
				session.put("devicesNodes", DevicesTreeMap);
				session.put("currentStateMap", currentStateMap);

				read = true;				
				
		}catch (Exception e){
			e.printStackTrace();
		}
		}
		
	}



//TEMPORAL
private StateImage getStateValue(Collection StateImageList){				
	Random rdm = new Random();
	int random = rdm.nextInt(StateImageList.size());
	return (StateImage)StateImageList.toArray()[random];
}

}
