/*
 * Copyright (C) 2010 Xisco Faulí Tarazona
 *
 * This file is part of Domestica.
 * 
 * Domestica is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * Domestica is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domestica; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
*/

package beans;  

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.faces.context.FacesContext;

import objects.Location;
import objects.SubLocation;
import objects.LocationDevice;

import org.primefaces.event.NodeSelectEvent;
import org.primefaces.model.DefaultTreeNode;
import org.primefaces.model.TreeNode;

public class Tree implements Serializable {  
    
    private TreeNode root;  
    private String selectedNode;
    private String buttonValue;
    private String scriptOnClick;
    private boolean renderWelcome;
  
    public Tree() { 

			Map session = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();

            Collection locationCol = (Collection)session.get("locations"); // Collection of Locations
            
            Map<String,TreeNode> locationNodes = (Map<String,TreeNode> )session.get("locationNodes"); // Map of location Nodes
            Map<String,TreeNode> deviceNodes =  (Map<String,TreeNode> )session.get("devicesNodes"); // Map of device Nodes
            
			root = new DefaultTreeNode("root",null);
			
			renderWelcome = true;

            Iterator iLocation = locationCol.iterator();
    		while (iLocation.hasNext()){
    				Location l = (Location)iLocation.next();
    				TreeNode parent = (TreeNode)locationNodes.get(l.getId());
    				if(l.getId().equals(l.getSuperLocation())){ // if id = parent's id it's root
    					root.addChild(parent);
    				}    				
    				if(!l.getSubLocation().isEmpty()){
    					Iterator isubLocation = l.getSubLocation().iterator();
    					while (isubLocation.hasNext()){
    						SubLocation sublocation = (SubLocation)isubLocation.next();
    						TreeNode child = (TreeNode)locationNodes.get(sublocation.getLocationIDRef());
    						parent.addChild(child);
    					}

    				}

    				if(!l.getLocationDevice().isEmpty()){
    					Iterator iDevices = l.getLocationDevice().iterator();
    					while(iDevices.hasNext()){
    						LocationDevice device = (LocationDevice)iDevices.next();
    						TreeNode child = (TreeNode)deviceNodes.get(device.getDeviceIDRef());  						
    						parent.addChild(child);						
    					}
    				}
    		}
    		updateButton();

    } 
  
    public void updateButton(){
		if(root.isExpanded()){
			buttonValue = "Collapse Tree";
			scriptOnClick = "mytree.collapseAll();";
			root.setExpanded(false);
		}
		else{
			buttonValue = "Expand Tree";
			scriptOnClick = "mytree.expandAll();";
			root.setExpanded(true);
		}
    }
    
    public String getButtonValue() {
    	return buttonValue;
	}


	public void setButtonValue(String buttonValue) {
		this.buttonValue = buttonValue;
	}


	public String getScriptOnClick() {
		
		return scriptOnClick;
	}


	public void setScriptOnClick(String scriptOnClick) {
		
		this.scriptOnClick = scriptOnClick;
	}


	public TreeNode getRoot() {  
        return root;  
    }  
      
    public String getSelectedNode() {  
        return selectedNode;  
    }  
  
    public void setSelectedNode(String selectedNode) {  
        this.selectedNode = selectedNode;  
    }  
    
    
    public boolean isRenderWelcome() {
		return renderWelcome;
	}


	public void setRenderWelcome(boolean renderWelcome) {
		this.renderWelcome = renderWelcome;
	}


	public void onNodeSelect(NodeSelectEvent event) { 
    	
		FacesContext context = FacesContext.getCurrentInstance();
		
		renderWelcome = false;
		
		LocationsPage img = (LocationsPage)context.getApplication().getExpressionFactory()
		.createValueExpression(context.getELContext(), "#{LocationsBean}", LocationsPage.class)
		.getValue(context.getELContext());
		
		DevicesPage inf = (DevicesPage)context.getApplication().getExpressionFactory()
		.createValueExpression(context.getELContext(), "#{DevicesBean}", DevicesPage.class)
		.getValue(context.getELContext());
		
    	if(event.getTreeNode().getType().equals("location")){
    		img.getImages(selectedNode);
    		inf.setPageRendered(false);
    	}
    	else if(event.getTreeNode().getType().equals("device")){
    		inf.getInformation(selectedNode);
    		img.setPageRendered(false);
    	}
    }
}